/*
 *  net/sf/logdistiller/FactoryMultiplexer.java
 *  Copyright (C) 2004-2007 Herv BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller;

import java.io.IOException;
import java.text.ParseException;
import java.util.*;

/**
 * Combines logevents fetched from multiple factories.
 * The resulting factory chooses the next logevent according to the order provided.
 */
public class FactoryMultiplexer extends LogEvent.Factory {
  private final Comparator logeventComparator;
  private final List factories = new ArrayList();
  private boolean initialized = false;
  private Comparator entryComparator;

  public FactoryMultiplexer(LogType.Description description) {
    this(description, null);
  }

  public FactoryMultiplexer(LogType.Description description, Comparator logeventComparator) {
    super(description, "multiplexer");
    this.logeventComparator = logeventComparator;
  }

  public void addFactory(LogEvent.Factory factory) {
    if (initialized) {
      throw new IllegalStateException("cannot add a new LogEvent.Factory after reading first event");
    }
    if (description != factory.getDescription()) {
      throw new IllegalArgumentException("cannot add a new LogEvent.Factory with different description");
    }
    factories.add(new Entry(factory));
  }

  protected LogEvent readNextEvent() throws IOException, ParseException {
    if (! initialized) {
      init();
    }
    if (factories.size() <= 0) {
      return null;
    }
    Entry entry = (Entry)factories.get(0);
    LogEvent nextEvent = entry.getEvent();
    if (entry.nextEvent() == null) {
      factories.remove(0);
    } else {
      sort();
    }
    return nextEvent;
  }

  /**
   * Initialize the multiplexer: fetch first event in each factory, remove empty
   * factories, init entry comparator and sort the factories.
   * @throws IOException
   * @throws ParseException
   */
  private void init() throws IOException, ParseException {
    initialized = true;
    LogEvent oneEvent = null;
    // fetch the first LogEvent
    List emptyFactories = new ArrayList();
    Iterator iter = factories.iterator();
    while (iter.hasNext()) {
      Entry entry = (Entry)iter.next();
      LogEvent event = entry.nextEvent();
      if (event == null) {
        emptyFactories.add(entry);
      } else {
        oneEvent = event;
      }
    }
    // remove empty factories
    factories.removeAll(emptyFactories);
    if (factories.size() >= 0) {
      // init entry comparator
      if (logeventComparator != null) {
        entryComparator = new EntrySpecificComparator(logeventComparator);
      } else if (oneEvent instanceof Comparable) {
        entryComparator = new EntryNaturalComparator();
      }
      // sort the factories
      sort();
    }
  }

  /**
   * Sort the factories, if necessary (a logevent comparator has been provided,
   * or logevent implements Comparable).
   */
  private void sort() {
    if (entryComparator != null) {
      Collections.sort(factories, entryComparator);
    }
  }

  private static class Entry {
    protected LogEvent nextEvent;
    private final LogEvent.Factory factory;
    public Entry(LogEvent.Factory factory) {
      this.factory = factory;
    }

    public LogEvent getEvent() {
      return nextEvent;
    }

    public LogEvent nextEvent() throws IOException, ParseException {
      nextEvent = factory.nextEvent();
      return nextEvent;
    }
  }

  /**
   * Comparator for entries with logevents that implement Comparable.
   */
  private static class EntryNaturalComparator implements Comparator {
    public int compare(Object object1, Object object2) {
      Entry entry1 = (Entry)object1;
      Entry entry2 = (Entry)object2;
      return ((Comparable)entry1.getEvent()).compareTo(entry2.getEvent());
    }
  }

  /**
   * Comparator for entries when a specific logevent Comparator is provided.
   */
  private static class EntrySpecificComparator implements Comparator {
    private final Comparator eventComparator;
    public EntrySpecificComparator(Comparator eventComparator) {
      this.eventComparator = eventComparator;
    }
    public int compare(Object object1, Object object2) {
      Entry entry1 = (Entry)object1;
      Entry entry2 = (Entry)object2;
      return eventComparator.compare(entry1.getEvent(), entry2.getEvent());
    }
  }
}
