/*
 *  net/sf/logdistiller/LogDistillation.java
 *  Copyright (C) 2004-2007 Herv BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller;

import java.io.*;
import java.util.*;

import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.ArrayUtils;

/**
 * The class executing a log distillation, using a log distiller, and processing every log events.
 * A log distillation can be run either with LogDistiller's {@link net.sf.logdistiller.gui GUI}
 * or with {@link net.sf.logdistiller.ant Ant tasks}.
 * @see LogDistiller
 */
public class LogDistillation implements Serializable {
  private static final long serialVersionUID = -9098523382541148543L;
  public final static String LINE_SEPARATOR = System.getProperty("line.separator");
  private final LogDistiller ld;
  private final Group[] groups;
  private final Category[] categories;
  private final String content;
  private final File destinationDirectory;
  private final String version = LogDistiller.getVersion();

  private transient Map mapGroups;
  private transient Map mapCategories;

  private long beginTime;
  private long endTime;
  private int eventCount = -1;
  private long eventBytes = 0;
  private int skippedEventCount = 0;
  private long skippedEventBytes = 0;

  public LogDistillation(LogDistiller ld) {
    this.ld = ld;
    LogDistiller.Category[] categoryDefinitions = ld.getCategories();
    int length = categoryDefinitions.length;
    categories = new Category[length];
    Map mapCategories = new HashMap();
    for (int i = 0; i < length; i++) {
      categories[i] = new Category(categoryDefinitions[i]);
      mapCategories.put(categoryDefinitions[i], categories[i]);
    }
    LogDistiller.Group[] groupDefinitions = ld.getGroups();
    length = groupDefinitions.length;
    groups = new Group[length];
    for (int i = 0; i < length; i++) {
      LogDistiller.Group groupDefinition = groupDefinitions[i];
      LogDistiller.Category categoryDefinition = groupDefinition.getCategory();
      Category category = (categoryDefinition == null) ? null : (Category)mapCategories.get(categoryDefinition);
      groups[i] = new Group(groupDefinition, category);
    }
    content = ld.getOutput().getContent();
    destinationDirectory = new File(ld.getOutput().getDirectory());
    updateRef();
  }

  private void updateRef() {
    String skip = ld.getOutput().getSkip();
    mapCategories = new HashMap();
    Category skipCategory = null;
    int len = categories.length;
    for (int i = 0; i < len; i++) {
      Category category = categories[i];
      mapCategories.put(category.getDefinition().getId(), category);
      category.updateRef(this);
      if (category.getDefinition().getId().equals(skip)) {
        skipCategory = category;
      }
    }
    mapGroups = new HashMap();
    len = groups.length;
    for (int i = 0; i < len; i++) {
      Group group = groups[i];
      mapGroups.put(group.getDefinition().getId(), group);
      group.updateRef(this);
      if (group.getDefinition().getId().equals(skip)) {
        group.setSkip(true);
      }
    }
    if (skipCategory != null) {
      skipCategory.setSkip(true);
    }
  }

  private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
    in.defaultReadObject();
    if (ld.getOutput() != null) {
      ld.updateRef(); // workaround: a problem with Group deserialization happens
      updateRef();
    }
  }

  /**
   * begin the distillation
 * @throws IOException 
   */
  public void begin() throws IOException {
    destinationDirectory.mkdirs();
    int count = groups.length;
    for (int i = 0; i < count; i++) {
      groups[i].begin(destinationDirectory);
    }
    eventCount = 0;
    beginTime = System.currentTimeMillis();
  }

  /**
   * Process a log event
   * @param le LogEvent the log event
   * @throws IOException if an error occurs when (eventually) saving the log event
   */
  public void processLogEvent(LogEvent le) throws IOException {
    if (eventCount < 0) {
      throw new IllegalStateException("LogDistiller not initialized: need to call begin() before processing");
    }
    eventCount++;
    int eventSize = le.getRawLog().length() + LINE_SEPARATOR.length();
    eventBytes += eventSize;
    int count = groups.length;
    for (int i = 0; i < count; i++) {
      Group group = groups[i];
      if (group.accept(le)) {
        if (group.isSkip()) {
          // logevent matched a skipped group => update counters
          eventCount--;
          eventBytes -= eventSize;
          skippedEventCount++;
          skippedEventBytes += eventSize;
          break;
        }
        if (! group.getDefinition().continueProcessing()) {
          break;
        }
      }
    }
  }

  /**
   * end the distillation
   * @throws IOException if an error occurs when closing the files where log events were (eventually) saved
   */
  public void end() throws IOException {
    endTime = System.currentTimeMillis();
    int count = groups.length;
    for (int i = 0; i < count; i++) {
      Group group = groups[i];
      group.end();
    }
    saveTo(newDestinationFile("result.ser"));
  }

  /**
   * Saves the distillation to a file, in Java serialized format.
   * @param file File destination file.
   */
  public void saveTo(File file) throws IOException {
    OutputStream out = null;
    try {
      out = new FileOutputStream(file);
      saveTo(out);
    } finally {
      IOUtils.closeQuietly(out);
    }
  }

  public void saveTo(OutputStream out) throws IOException {
    ObjectOutputStream obj = new ObjectOutputStream(out);
    obj.writeObject(this);
  }

  public List listCategoriesToReport() {
    List reportCategories = new ArrayList();
    int count = categories.length;
    for (int i = 0; i < count; i++) {
      Category category = categories[i];
      if ((! category.isSkip()) && (category.sumEventCount() > 0)) {
        reportCategories.add(category);
      }
    }
    return reportCategories;
  }

  public List listGroupsToReport() {
    List reportGroups = new ArrayList();
    int count = groups.length;
    for (int i = 0; i < count; i++) {
      Group group = groups[i];
      if ((! group.isSkip()) && (group.getEventCount() > 0) && (group.getCategory() == null)) {
        reportGroups.add(group);
      }
    }
    return reportGroups;
  }

  public LogDistiller getDefinition() {
    return ld;
  }

  public Group[] getGroups() {
    return (Group[])ArrayUtils.clone(groups);
  }

  public Group getGroup(String id) {
    return (Group)mapGroups.get(id);
  }

  public Category[] getCategories() {
    return (Category[])ArrayUtils.clone(categories);
  }

  public Category getCategory(String id) {
    return (Category)mapCategories.get(id);
  }

  public String getContent() {
    return content;
  }

  public int getEventCount() {
    return eventCount;
  }

  public long getEventBytes() {
    return eventBytes;
  }

  public long getBeginTime() {
    return beginTime;
  }

  public long getEndTime() {
    return endTime;
  }

  public int getSkippedEventCount() {
    return skippedEventCount;
  }

  public long getSkippedEventBytes() {
    return skippedEventBytes;
  }

  public File newDestinationFile(String filename) {
    return new File(destinationDirectory, filename);
  }

  /**
   * What is the version of <b>LogDistiller</b> that was used to create this instance?
   * @return String the version
   */
  public String getVersion() {
    return (version == null) ? "unknown" : version;
  }

  public class Group implements Serializable {
    private static final long serialVersionUID = 5788707827350407084L;
    private final LogDistiller.Group definition;
    private final Category category;

    private transient LogDistillation logdistillation;

    private boolean skip = false;

    /** maximum number of events saved, -1 = no limit */
    private final int maxSaveCount;
    /** maximum size sum of events saved in bytes, -1 = no limit */
    private final long maxSaveBytes;
    private final Plugin[] plugins;
    private File logFile;
    private transient PrintStream logStream;
    private int eventCount = 0;
    private int savedEventCount = 0;
    private long bytes = 0;

    public Group(LogDistiller.Group definition, Category category) {
      this.definition = definition;
      this.category = category;
      maxSaveCount = Integer.parseInt(definition.getParam("maxSave.count", "-1"));
      maxSaveBytes = Long.parseLong(definition.getParam("maxSave.size", "-1")) * 1024;
      LogDistiller.Plugin[] pluginDefs = definition.getPlugins();
      plugins = new Plugin[pluginDefs.length];
      for (int i = 0; i < pluginDefs.length; i++) {
        plugins[i] = Plugins.newInstance(pluginDefs[i]);
      }
    }

    private void updateRef(LogDistillation logdistillation) {
      this.logdistillation = logdistillation;
      if (category != null) {
        category.addGroup(this);
      }
      int len = plugins.length;
      for (int i = 0; i < len; i++) {
        plugins[i].updateRef(this);
      }
    }

    public LogDistillation getLogdistillation() {
      return logdistillation;
    }

    public Category getCategory() {
      return category;
    }

    public void setSkip(boolean skip) {
      this.skip = skip;
    }

    public boolean isSkip() {
      return skip;
    }

    /**
     * Check if given LogEvent meet one of the conditions defined.
     * @param le LogEvent the LogEvent to check
     * @throws IOException
     * @return boolean <code>true</code> if the LogEvent meets a condition
     */
    public boolean accept(LogEvent le) throws IOException {
      if (definition.accept(le)) {
        addLogEvent(le);
        return true;
      }
      return false;
    }

    /**
     * Add a LogEvent to the current Group.
     * @param le LogEvent
     * @throws IOException
     */
    protected void addLogEvent(LogEvent le) throws IOException {
      eventCount++;
      bytes += le.getRawLog().length() + LINE_SEPARATOR.length();
      if (definition.getSave()
          && ((maxSaveCount < 0) || (eventCount <= maxSaveCount))
          && ((maxSaveBytes < 0) || (bytes <= maxSaveBytes))) {
        if (logStream == null) {
          logStream = new PrintStream(new FileOutputStream(logFile));
        }
        logStream.println(le.getRawLog());
        savedEventCount++;
      }
      int len = plugins.length;
      for (int i = 0; i < len; i++) {
        plugins[i].addLogEvent(le);
      }
    }

    void begin(File destinationDirectory) throws IOException {
      logFile = newDestinationFile(definition.getId() + ".log");
      int len = plugins.length;
      for (int i = 0; i < len; i++) {
        plugins[i].begin(destinationDirectory);
      }
    }

    void end() throws IOException {
      if (logStream != null) {
        logStream.close();
      }
      int len = plugins.length;
      for (int i = 0; i < len; i++) {
        plugins[i].end();
      }
    }

    public LogDistiller.Group getDefinition() {
      return definition;
    }

    public int getEventCount() {
      return eventCount;
    }

    public int getSavedEventCount() {
      return savedEventCount;
    }

    public long getBytes() {
      return bytes;
    }

    public File getLogFile() {
      return logFile;
    }

    public int getMaxSaveCount() {
      return maxSaveCount;
    }

    public long getMaxSaveBytes() {
      return maxSaveBytes;
    }

    public Plugin[] getPlugins() {
      return (Plugin[])ArrayUtils.clone(plugins);
    }
  }

  public static class Category implements Serializable {
    private static final long serialVersionUID = 7098883103191641753L;
    private final LogDistiller.Category definition;

    private transient LogDistillation logdistillation;
    private transient List groups;

    private boolean skip = false;

    public Category(LogDistiller.Category definition) {
      this.definition = definition;
    }

    private void updateRef(LogDistillation logdistillation) {
      this.logdistillation = logdistillation;
      groups = new ArrayList();
    }

    public LogDistillation getLogdistillation() {
      return logdistillation;
    }

    public LogDistiller.Category getDefinition() {
      return definition;
    }

    private void addGroup(Group group) {
      groups.add(group);
    }

    public List getGroups() {
      return groups;
    }

    public void setSkip(boolean skip) {
      this.skip = skip;
      Iterator iter = groups.iterator();
      while (iter.hasNext()) {
        Group group = (Group)iter.next();
        group.setSkip(skip);
      }
    }

    public boolean isSkip() {
      return skip;
    }

    public int sumEventCount() {
      int sum = 0;
      Iterator iter = groups.iterator();
      while (iter.hasNext()) {
        Group group = (Group)iter.next();
        sum += group.getEventCount();
      }
      return sum;
    }

    public long sumBytes() {
      long sum = 0;
      Iterator iter = groups.iterator();
      while (iter.hasNext()) {
        Group group = (Group)iter.next();
        sum += group.getBytes();
      }
      return sum;
    }

    public List listGroupsToReport() {
      List reportGroups = new ArrayList();
      Iterator iter = groups.iterator();
      while (iter.hasNext()) {
        Group group = (Group)iter.next();
        if (group.getEventCount() > 0) {
          reportGroups.add(group);
        }
      }
      return reportGroups;
    }
  }

  public static abstract class Plugin implements Serializable {
    private static final long serialVersionUID = -722318023109374109L;
    protected final LogDistiller.Plugin definition;

    protected transient Group group;

    public Plugin(LogDistiller.Plugin definition) {
      this.definition = definition;
    }

    private void updateRef(Group group) {
      this.group = group;
    }

    public Group getGroup() {
      return group;
    }

    public LogDistiller.Plugin getDefinition() {
      return definition;
    }

    abstract public void begin(File destinationDirectory) throws IOException;

    abstract public void addLogEvent(LogEvent le) throws IOException;

    abstract public void end() throws IOException;

    abstract public void appendGroupReport(ReportFormat.PluginReport report);

    abstract public void appendGlobalReport(ReportFormat.PluginReport report);
  }
}
