/*
 *  net/sf/logdistiller/LogEvent.java
 *  Copyright (C) 2004-2007 Herv BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.text.ParseException;

import org.apache.commons.lang.ArrayUtils;

/**
 * Log events base class. Every new log type must implement an inherited class,
 * with appropriate:<ul>
 * <li>{@link Factory} to create events from the log stream</li>
 * <li>{@link LogType} to declare the log type and eventually available parameters to customize its format</li>
 * </ul>
 * @see LogType
 * @see LogTypes
 */
public abstract class LogEvent {
  private final Factory factory;
  private final String rawLog;
  private String[] attributes;

  protected LogEvent(Factory factory, String rawLog) {
    this.factory = factory;
    this.rawLog = rawLog;
  }

  protected void setAttributes(String[] attributes) {
    int count = factory.getDescription().getAttributesCount();
    if (attributes.length != count) {
      throw new IllegalArgumentException("expected " + count + " attributes, got " + attributes.length);
    }
    this.attributes = attributes;
  }

  protected void checkInitialized() {
    if (attributes == null) {
      throw new IllegalStateException("log event not initialized: call setAttributes() to initialize it");
    }
  }

  /**
   * get the raw log text from which this event was parsed.
   * @return String
   */
  public String getRawLog() {
    return rawLog;
  }

  public String getAttribute(int pos) {
    checkInitialized();
    return attributes[pos];
  }

  public String[] getAttributes() {
    checkInitialized();
    return (String[])ArrayUtils.clone(attributes);
  }

  public int getAttributesCount() {
    checkInitialized();
    return attributes.length;
  }

  public String getAttribute(String attributeName) {
    int index = factory.getDescription().getAttributeIndex(attributeName);
    if (index < 0) {
      throw new IllegalArgumentException("unknown attribute '" + attributeName + "'");
    }
    return getAttribute(index);
  }

  public Factory getFactory() {
    return factory;
  }

  public void dump(PrintWriter out) {
    out.println("- raw log: " + getRawLog());
    for (int i = 0; i < getAttributesCount(); i++) {
      out.println("- " + getFactory().getDescription().getAttributeName(i) + ": " + getAttribute(i));
    }
  }

  public String dump() {
    StringWriter sw = new StringWriter();
    PrintWriter out = new PrintWriter(sw, false);
    dump(out);
    out.close();
    return sw.toString();
  }

  /**
   * Get the timestamp.
   * @return the timestamp or "-" if no this log type does not have a timestamp
   * @see LogType.Description#getTimestampAttribute()
   */
  public String getTimestamp() {
    int index = factory.getDescription().getTimestampAttribute();
    return (index > 0) ? getAttribute(index) : "-";
  }

  /**
   * The base class for LogEvent factories, responsible for parsing an input
   * stream into <code>LogEvent</code>s.
   */
  public static abstract class Factory {
    public final static String NEWLINE = System.getProperty("line.separator");
    protected final LogType.Description description;
    protected final String logSource;
    private LogEvent lastEvent;
    private LogEvent pushedbackEvent;

    protected Factory(LogType.Description description, String logSource) {
      this.description = description;
      this.logSource = logSource;
    }

    /**
     * gets the next LogEvent.
     * @throws IOException
     * @throws ParseException
     * @return LogEvent the new LogEvent read, or <code>null</code> if none available any more
     */
    public LogEvent nextEvent() throws IOException, ParseException {
      LogEvent nextEvent = null;
      if (pushedbackEvent != null) {
        nextEvent = pushedbackEvent;
        pushedbackEvent = null;
      }
      else {
        nextEvent = readNextEvent();
      }
      lastEvent = nextEvent;
      return nextEvent;
    }

    public void pushbackLastEvent() throws IllegalStateException {
      if (pushedbackEvent != null) {
        throw new IllegalStateException("The last event has already been pushed back");
      }
      if (lastEvent == null) {
        throw new IllegalStateException("There is no last event to push back");
      }
      pushedbackEvent = lastEvent;
    }

    public LogType.Description getDescription() {
      return description;
    }

    public String getLogSource() {
      return logSource;
    }

    protected abstract LogEvent readNextEvent()
        throws IOException, ParseException;
  }
}
