/*
 *  net/sf/logdistiller/LogType.java
 *  Copyright (C) 2004-2007 Herv� BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller;

import java.io.IOException;
import java.io.Reader;
import java.util.*;

/**
 * Base class for log type implementation.
 * A log type identifies the log format, which can be customized depending on parameter's values.
 * When defining a new log type, {@link #newDescription newDescription(Map params)} method has
 * to be implemented to define and handle log type's parameters.
 * @see LogType.Basic
 * @see LogType.Description
 * @see LogTypes
 */
public abstract class LogType {
  private final String id;

  protected LogType(String id) {
    this.id = id;
  }

  /**
   * Create a new log description customized with parameters.
   * @param params the parameters to customize the log type definition
   */
  public abstract Description newDescription(Map params);

  public String getId() {
    return id;
  }

  /**
   * Basic logtype definition that does not provide any parameters for customization. It always returns the same
   * Description instance, which must have been set by calling {@link #setDescription}.
   */
  public static class Basic extends LogType {
    private Description description;
    public Basic(String id) {
      super(id);
    }

    public void setDescription(Description description) {
      this.description = description;
    }

    public Description newDescription(Map params) {
      if (description == null) {
        throw new IllegalStateException("description must be set before basic logtype use");
      }
      return description;
    }
  }

  /**
   * Description of a log format corresponding to a log type. It must be subclassed for every
   * log type to implement specialized {@link #newFactory(Reader, String)} method.
   */
  public static abstract class Description {
    private final LogType logtype;
    private final List attributeNames;

    /**
     * Create a new LogType description. By convention, the first attribute should be <code>logSource</code> and
     * the second should be the timestamp (if this notion is relevant for the log type).
     * @param logtype
     * @param attributeNames
     * @see #getTimestampAttribute()
     * @see LogEvent#getTimestamp()
     */
    public Description(LogType logtype, String[] attributeNames) {
      this.logtype = logtype;
      this.attributeNames = Collections.unmodifiableList(Arrays.asList(attributeNames));
    }

    public LogType getLogType() {
      return logtype;
    }

    public List getAttributeNames() {
      return attributeNames;
    }

    public int getAttributesCount() {
      return attributeNames.size();
    }

    public String getAttributeName(int pos) {
      return (String)attributeNames.get(pos);
    }

    /**
     * Get the attribute that represents the timestamp.
     * @return the attribute count, or <code>-1</code> if this logtype does not have a timestamp
     */
    public int getTimestampAttribute() {
      return 1;
    }

    /**
     * Get the group definitions specific to this log type that will be added to the generated
     * rules configuration generated by the GUI.
     * @return an XML fragment containing group definitions
     */
    public String getDefaultSpecificGroups() {
      return "";
    }

    /**
     * Get the attributes that can be used as a sorting key to get useful event sampling in the last "unknown" group
     * (separated with a quote when multiple attributes).
     * They will be used as a default sampling rule configuration.
     * @return "attr1[,attr2[,...]]"
     */
    public String getDefaultSamplingAttributes() {
      return "";
    }

    /**
     * Find the attribute index corresponding to an attribute name.
     * @param name String the attribute name to search for
     * @return int the index of the attribute, or -1 if not found
     */
    public int getAttributeIndex(String name) {
      return attributeNames.indexOf(name);
    }

    /**
     * Create a new LogEvent factory for a log stream corresponding to this log description.
     * @param reader the stream containing to parse
     * @param logSource a textual description of the stream's source (useful when LogDistiller works on multiple
     * simultaneous log sources)
     */
    public abstract LogEvent.Factory newFactory(Reader reader, String logSource) throws IOException;
  }
}
