/*
 *  net/sf/logdistiller/LogTypes.java
 *  Copyright (C) 2004-2007 Herv BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller;

import java.util.*;

import net.sf.logdistiller.logtypes.BaseLogTypes;
import net.sf.logdistiller.util.ExtensionHelper;
import net.sf.logdistiller.util.FormatUtil;

/**
 * Definition of available log types. By default, some log types come {@link net.sf.logdistiller.logtypes.BaseLogTypes
 * predefined} with <b>LogDistiller</b>. You can add custom log types: <ul>
 * <li>1. create a new <code>LogEvent</code> class, with corresponding <code>LogEvent.Factory</code> and
 * <code>LogType</code></li>
 * <li>2. create a new <code>LogTypes</code> class defining previous <code>LogType</code> with a default constructor</li>
 * <li>3. create a <code>logdistiller.properties</code> resource file which <code>logtypes</code> property is
 * the full <code>LogTypes</code> class name</li>
 * </ul>
 * @see LogType
 */
public abstract class LogTypes {
  /**
   * get the <code>LogType</code> instances defined by this extension.
   * @return the list of logtypes
   * @see LogType
   */
  public abstract List defineLogTypes();

  public static final List ALL;
  public static final Map MAP;
  static {
    List all = new ArrayList(new BaseLogTypes().defineLogTypes());
    all.addAll(loadExtensionLogTypes());
    ALL = Collections.unmodifiableList(all);

    Map map = new HashMap();
    Iterator iter = ALL.iterator();
    while (iter.hasNext()) {
      LogType type = (LogType)iter.next();
      map.put(type.getId(), type);
    }
    MAP = Collections.unmodifiableMap(map);
  }

  /**
   * get all the log types defined (predefined and custom).
   */
  public static List getAllLogTypes() {
    return ALL;
  }

  public static String listAllLogTypeIds() {
    return FormatUtil.join(", ", MAP.keySet().iterator());
  }

  public static LogType getLogType(String id) {
    return (LogType)MAP.get(id);
  }

  /**
   * Loads logtypes defined by extension mechanism: if defined, <code>logtypes</code> property in
   * <code>logdistiller.properties</code> is the full class name of a concrete implementation of <code>LogTypes</code>
   * class.
   * @return List the list of every <code>LogType</code> defined by loaded <code>LogTypes</code>
   */
  private static List loadExtensionLogTypes() {
    Iterator iter = ExtensionHelper.findExtensions("logtypes").iterator();
    List logtypes = new ArrayList();
    while (iter.hasNext()) {
      String logtypesClass = (String)iter.next();
      if (logtypesClass != null) {
        logtypes.addAll(loadLogTypes(logtypesClass));
      }
    }
    return logtypes;
  }

  private static List loadLogTypes(String logtypesClass) {
    try {
      LogTypes logtypes = (LogTypes) Class.forName(logtypesClass).newInstance();
      return logtypes.defineLogTypes();
    } catch (ClassNotFoundException cnfe) {
      throw new RuntimeException("unable to load LogTypes subclass " + logtypesClass, cnfe);
    } catch (IllegalAccessException iae) {
      throw new RuntimeException("unable to access LogTypes subclass constructor for " + logtypesClass, iae);
    } catch (InstantiationException ie) {
      throw new RuntimeException("unable to instanciate LogTypes subclass " + logtypesClass, ie);
    }
  }
}
