/*
 *  net/sf/logdistiller/Match.java
 *  Copyright (C) 2004-2007 Herv BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller;

import java.io.*;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Pattern;

import org.jdom.Element;

/**
 * Match of an attribute of a LogEvent. Match type can be either
 * <code>contains</code>, <code>equals</code>, <code>startsWith</code>,
 * <code>endsWith</code> or <code>regexp</code>, and can be negated by preceding
 * type with <code>!</code>.
 */
public class Match implements Serializable {
  private static final long serialVersionUID = -1809458932445222708L;
  private final String attribute;
  private final boolean not;
  private final Function function;

  public Match(String attribute, String type, String reference) {
    this.attribute = attribute;
    this.not = type.startsWith("!");
    if (not) {
      type = type.substring(1);
    }
    this.function = Match.Function.getInstance(type, reference);
  }

  public boolean match(LogEvent le) {
    String value = le.getAttribute(attribute);
    boolean result = function.checkValue(value);
    return (not ? !result : result);
  }

  public Element dump() {
    Element elmt = new Element("match");
    elmt.setAttribute("attribute", attribute);
    elmt.setAttribute("type", (not ? "!" : "") + function.getId());
    elmt.setText(function.reference);
    return elmt;
  }

  private static abstract class Function implements Serializable {
    private static final long serialVersionUID = 1072441995042764838L;
    public final static Map FUNCTIONS;
    static {
      Map map = new HashMap();
      map.put(Contains.ID, Contains.class);
      map.put(Equals.ID, Equals.class);
      map.put(StartsWith.ID, StartsWith.class);
      map.put(EndsWith.ID, EndsWith.class);
      map.put(Regexp.ID, Regexp.class);
      FUNCTIONS = Collections.unmodifiableMap(map);
    }

    public static Function getInstance(String type, String reference) {
      Class functionClass = (Class)FUNCTIONS.get(type);
      if (functionClass == null) {
        throw new IllegalArgumentException("function type not supported '" + type + "'");
      }
      try {
        Function function = (Function) functionClass.newInstance();
        function.setReference(reference);
        return function;
      } catch (IllegalAccessException iae) {
        // can't happen
        throw new RuntimeException("an impossible condition has arrived...", iae);
      } catch (InstantiationException ie) {
        // can't happen
        throw new RuntimeException("an impossible condition has arrived...", ie);
      }
    }

    protected String reference;

    public void setReference(String reference) {
      this.reference = reference;
    }

    public abstract String getId();

    /**
     * check if the given value matches the reference.
     * @param value String the value to compare to reference
     * @return boolean
     */
    public abstract boolean checkValue(String value);
  }

  protected static class Contains extends Function {
    private static final long serialVersionUID = 4613611844726287358L;
    public static final String ID = "contains";
    public String getId() {
      return ID;
    }
    public boolean checkValue(String value) {
      return (value.indexOf(reference) >= 0);
    }
  }
  protected static class Equals extends Function {
    private static final long serialVersionUID = -6539918940865536415L;
    public static final String ID = "equals";
    public String getId() {
      return ID;
    }
    public boolean checkValue(String value) {
      return reference.equals(value);
    }
  }
  protected static class StartsWith extends Function {
    private static final long serialVersionUID = 5014914695776363132L;
    public static final String ID = "startsWith";
    public String getId() {
      return ID;
    }
    public boolean checkValue(String value) {
      return value.startsWith(reference);
    }
  }
  protected static class EndsWith extends Function {
    private static final long serialVersionUID = 1819477635452601804L;
    public static final String ID = "endsWith";
    public String getId() {
      return ID;
    }
    public boolean checkValue(String value) {
      return value.endsWith(reference);
    }
  }

  protected static class Regexp extends Function {
    private static final long serialVersionUID = -2067188055591690252L;
    public static final String ID = "regexp";
    public String getId() {
      return ID;
    }
    private transient Pattern pattern;
    public void setReference(String reference) {
      super.setReference(reference);
      updatePattern();
    }
    private void updatePattern() {
      pattern = Pattern.compile(reference);
    }
    private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
      in.defaultReadObject();
      updatePattern();
    }
    public boolean checkValue(String value) {
      return pattern.matcher(value).find();
    }
  }
}
