/*
 *  net/sf/logdistiller/Plugins.java
 *  Copyright (C) 2004-2007 Herv BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller;

import java.util.*;

import net.sf.logdistiller.plugins.BasePlugins;
import net.sf.logdistiller.util.ExtensionHelper;
import net.sf.logdistiller.util.FormatUtil;

/**
 * Definition of available plugins. By default, some plugins come {@link net.sf.logdistiller.plugins.BasePlugins
 * predefined} with <b>LogDistiller</b>. You can add custom plugins: <ul>
 * <li>1. create a new <code>LogDistillation.Plugin</code> class</li>
 * <li>2. create a new <code>Plugins</code> class defining previous <code>Plugin</code> with a default constructor</li>
 * <li>3. create a <code>logdistiller.properties</code> resource file which <code>plugins</code> property is
 * the full <code>Plugins</code> class name</li>
 * </ul>
 */
public abstract class Plugins {
  /**
   * get the <code>Plugin</code> instances defined by this extension.
   * @return the list of plugins
   * @see Plugin
   */
  public abstract List definePlugins();

  public static final List ALL;
  public static final Map MAP;
  static {
    List all = new ArrayList(new BasePlugins().definePlugins());
    all.addAll(loadExtensionPlugins());
    ALL = Collections.unmodifiableList(all);

    Map map = new HashMap();
    Iterator iter = ALL.iterator();
    while (iter.hasNext()) {
      Plugin plugin = (Plugin)iter.next();
      map.put(plugin.getId(), plugin);
    }
    MAP = Collections.unmodifiableMap(map);
  }

  /**
   * get all the plugins defined (predefined and custom).
   */
  public static List getAllPlugins() {
    return ALL;
  }

  public static String listAllPluginIds() {
    return FormatUtil.join(", ", MAP.keySet().iterator());
  }

  public static Plugin getPlugin(String id) {
    return (Plugin)MAP.get(id);
  }

  /**
   * Instanciates a new distillation plugin from its distiller configuration.
   * @param pluginConf the plugin distiller configuration
   * @return the distillation plugin
   * @throws PluginConfigException if the plugin type defined in the configuration does not exist
   */
  public static LogDistillation.Plugin newInstance(LogDistiller.Plugin pluginConf) {
    Plugin plugin = getPlugin(pluginConf.getType());
    if (plugin == null) {
      throw new PluginConfigException("plugin type '" + pluginConf.getType() + "' unknown, valid value: " + listAllPluginIds());
    }
    return plugin.newInstance(pluginConf);
  }

  /**
   * Loads plugins defined by extension mechanism: if defined, <code>plugins</code> property in
   * <code>logdistiller.properties</code> is the full class name of a concrete implementation of <code>Plugins</code>
   * class.
   * @return List the list of every <code>Plugin</code> defined by loaded <code>Plugins</code>
   */
  private static List loadExtensionPlugins() {
    Iterator iter = ExtensionHelper.findExtensions("plugins").iterator();
    List plugins = new ArrayList();
    while (iter.hasNext()) {
      String pluginsClass = (String)iter.next();
      if (pluginsClass != null) {
        plugins.addAll(loadPlugins(pluginsClass));
      }
    }
    return plugins;
  }

  private static List loadPlugins(String pluginsClass) {
    try {
      Plugins plugins = (Plugins) Class.forName(pluginsClass).newInstance();
      return plugins.definePlugins();
    } catch (ClassNotFoundException cnfe) {
      throw new RuntimeException("unable to load plugins class " + pluginsClass, cnfe);
    } catch (IllegalAccessException iae) {
      throw new RuntimeException("unable to access plugins constructor for class " + pluginsClass, iae);
    } catch (InstantiationException ie) {
      throw new RuntimeException("unable to instanciate plugins class " + pluginsClass, ie);
    }
  }
}
