/*
 *  net/sf/logdistiller/Publishers.java
 *  Copyright (C) 2004-2007 Herv BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller;

import java.util.*;

import net.sf.logdistiller.publishers.BasePublishers;
import net.sf.logdistiller.util.ExtensionHelper;
import net.sf.logdistiller.util.FormatUtil;

/**
 * Definition of available report publishers.
 */
public abstract class Publishers {
  /**
   * get the <code>Publishers</code> instances defined by this extension.
   * @return the list of publishers
   * @see LogType
   */
  public abstract List definePublishers();

  public static final List ALL;
  public static final Map MAP;
  static {
    List all = new ArrayList(new BasePublishers().definePublishers());
    all.addAll(loadExtensionPublishers());
    ALL = Collections.unmodifiableList(all);

    Map map = new HashMap();
    Iterator iter = ALL.iterator();
    while (iter.hasNext()) {
      Publisher type = (Publisher)iter.next();
      map.put(type.getId(), type);
    }
    MAP = Collections.unmodifiableMap(map);
  }

  /**
   * get all the report publishers defined (predefined and custom).
   */
  public static List getAllPublishers() {
    return ALL;
  }

  public static String listAllPublishersIds() {
    return FormatUtil.join(", ", MAP.keySet().iterator());
  }

  public static Publisher getPublisher(String id) {
    return (Publisher)MAP.get(id);
  }

  /**
   * Loads report publishers defined by extension mechanism: if defined, <code>publishers</code> property in
   * <code>logdistiller.properties</code> is the full class name of a concrete implementation of <code>Publishers</code>
   * class.
   * @return List the list of every <code>Publisher</code> defined by loaded <code>Publishers</code>
   */
  private static List loadExtensionPublishers() {
    Iterator iter = ExtensionHelper.findExtensions("publishers").iterator();
    List publishers = new ArrayList();
    while (iter.hasNext()) {
      String publishersClass = (String)iter.next();
      if (publishersClass != null) {
        publishers.addAll(loadPublishers(publishersClass));
      }
    }
    return publishers;
  }

  private static List loadPublishers(String publishersClass) {
    try {
      Publishers publishers = (Publishers) Class.forName(publishersClass).newInstance();
      return publishers.definePublishers();
    } catch (ClassNotFoundException cnfe) {
      throw new RuntimeException("unable to load plugins class " + publishersClass, cnfe);
    } catch (IllegalAccessException iae) {
      throw new RuntimeException("unable to access plugins constructor for class " + publishersClass, iae);
    } catch (InstantiationException ie) {
      throw new RuntimeException("unable to instanciate plugins class " + publishersClass, ie);
    }
  }
}
