/*
 *  net/sf/logdistiller/ant/LogDistillerTask.java
 *  Copyright (C) 2004-2007 Herv� BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller.ant;

import java.io.*;
import java.text.*;
import java.util.*;
import java.util.zip.*;
import javax.xml.parsers.*;

import org.apache.tools.ant.*;
import org.apache.tools.ant.types.*;
import org.xml.sax.*;

import net.sf.logdistiller.*;
import net.sf.logdistiller.util.*;
import net.sf.logdistiller.xml.*;

/**
 * Ant task to configure and launch a log distillation.
 * <pre>&lt;logdistiller conf="<i>[xml configuration file]</i>" publish="<i>[all|file,mail,...]</i>" compressed="<i>[no|gzip|zip]</i>"&gt;
 *  [&lt;<a href="AntProperty.html">property</a> .../&gt;]*
 *  &lt;fileset ...&gt;
 *&lt;/logdistiller&gt;</pre>
 */
public class LogDistillerTask extends Task {
  private File conf;
  private boolean mail = true;
  private boolean mailConfigured = false;
  private List publish;
  private boolean publishConfigured = false;
  private Map properties = new HashMap();
  private AntLogEvent logevent;
  private AntLogDistillation logdistillation;
  private List filesets = new ArrayList();
  private String compressed = "no";

  public LogDistillerTask() {
  }

  public void setConf(File conf) {
    this.conf = conf;
  }

  public void setMail(boolean mail) {
    this.mail = mail;
    mailConfigured = true;
  }

  public void setPublish(String publish) {
    publishConfigured = true;
    publish = publish.trim();
    if (publish.equals("all")) {
      this.publish = null;
    } else {
      this.publish = new ArrayList();
      while (publish.length() > 0) {
        int index = publish.indexOf(',');
        if (index > -1) {
          this.publish.add(publish.substring(0, index).trim());
          publish = publish.substring(index + 1);
        } else {
          this.publish.add(publish);
          publish = "";
        }
      }
    }
  }

  public void addConfiguredProperty(AntProperty property) {
    properties.put(property.getName(), property.getValue());
  }

  public void setCompressed(Compressed attr) {
    this.compressed = attr.getValue();
  }


  /**
   * @deprecated since LogDistiller 0.7, replaced by logtype element in rules configuration
   */
  public AntLogEvent createLogevent() {
    logevent = new AntLogEvent(getProject());
    return logevent;
  }

  /**
   * @deprecated since LogDistiller 0.7, replaced by output element in rules configuration
   */
  public AntLogDistillation createLogdistillation() {
    logdistillation = new AntLogDistillation();
    return logdistillation;
  }

  public void addFileSet(FileSet fs) {
    filesets.add(fs);
  }

  public void execute() throws BuildException {
    log("LogDistiller " + LogDistiller.getVersion());
    LogDistiller ld = readLogDistiller();
    compatibilityUpdate(ld);

    LogEvent.Factory factory = null;
    LogType logtype = LogTypes.getLogType(ld.getLogType().getId());
    if (logtype == null) {
      throw new BuildException("Could not find logtype '" + ld.getLogType().getId() + "': available values are " +
                               LogTypes.listAllLogTypeIds());
    }
    LogType.Description description = logtype.newDescription(ld.getLogType().getParams());
    ld.checkLogtypeAttributes(description);
    factory = initLogEventFactory(description);

    log("content: " + ld.getOutput().getContent() + ", output directory: " + ld.getOutput().getDirectory());
    LogDistillation distillation = new LogDistillation(ld);
    try {
      distillation.begin();
    } catch (IOException ioe) {
      throw new BuildException("I/O error when beginning logdistillation: " + ioe.getMessage(), ioe);
    }

    int count = 0;
    try {
      LogEvent le;
      while ((le = factory.nextEvent()) != null) {
        count++;
        distillation.processLogEvent(le);
      }
    } catch (ParseException pe) {
      throw new BuildException("parsing error when reading a log event: " + pe.getMessage(), pe);
    } catch (IOException ioe) {
      throw new BuildException("I/O error when reading a log event: " + ioe.getMessage(), ioe);
    }

    try {
      distillation.end();
      log("logevents distilled: " + count + ", time elapsed: " +
          FormatUtil.formatPeriod(distillation.getEndTime() - distillation.getBeginTime()));
    } catch (IOException ioe) {
      throw new BuildException("I/O error when ending logdistillation: " + ioe.getMessage(), ioe);
    }

    try {
      int reports = new PublishHelper(distillation, publish).publish();
      log("reports published: " + reports);
    } catch (IOException ioe) {
      throw new BuildException("I/O error when publishing logdistillation reports: " + ioe.getMessage(), ioe);
    } catch (Publisher.PublishException ppe) {
      throw new BuildException("Error when publishing logdistillation reports: " + ppe.getMessage(), ppe);
    }
  }

  /**
   * Make all that's possible to update latest LogDistiller model to maintain compatibility with older versions, since
   * some elements have moved from Ant task to LogDistiller model.
   * @param ld LogDistiller
   */
  private void compatibilityUpdate(LogDistiller ld) {
    if (logevent != null) {
      if (ld.isOld()) {
        String warning = "element 'logevent' is deprecated in Ant logdistiller task, please use element 'logtype' " +
            "from " + LogDistillerEntityResolver.LATEST_DTD + " in your rules file";
        log("WARNING: " + warning, Project.MSG_WARN);
        ld.addWarning(warning);
        ld.buildCompatibilityLogType(logevent.getLogtype());
      } else {
        String warning = "element 'logevent' is deprecated in Ant logdistiller task now that you use element " +
            "'logtype' from " + LogDistillerEntityResolver.LATEST_DTD + " in your rules file: ignored (please remove)";
        log("WARNING : " + warning, Project.MSG_WARN);
        ld.addWarning(warning);
      }
    }
    if (logdistillation != null) {
      if (ld.isOld()) {
        String warning = "element 'logdistillation' is deprecated in Ant logdistiller task, please use element " +
            "'output' from " + LogDistillerEntityResolver.LATEST_DTD + " in your rules file";
        log("WARNING: " + warning, Project.MSG_WARN);
        ld.addWarning(warning);
        String skipgroups = logdistillation.getSkipgroups();
        int pos = skipgroups.indexOf(',');
        if (pos >= 0) {
          skipgroups = skipgroups.substring(0, pos).trim();
          warning = "attribute skipgroups in logdistillation element cannot refer any more to multiple groups: " +
              "please create a category. Only group '" + skipgroups + "' has been kept.";
        }
        ld.getOutput().updateCompatibility(logdistillation.getDestdir(), logdistillation.getContent(), skipgroups);
      } else {
        String warning = "element 'logdistillation' is deprecated in Ant logdistiller task now that you use element " +
            "'output' from " + LogDistillerEntityResolver.LATEST_DTD + " in your rules file: ignored (please remove)";
        log("WARNING: " + warning, Project.MSG_WARN);
        ld.addWarning(warning);
      }
    }
    if (mailConfigured) {
      if (! publishConfigured) {
        setPublish(mail ? "file,mail" : "file");
      } else {
        String warning = "attribute 'mail' is deprecated in Ant logdistiller task now that you use attribute " +
            "'publish': attribute ignored (please remove)";
        log("WARNING: " + warning, Project.MSG_WARN);
        ld.addWarning(warning);
      }
    }
  }

  private LogDistiller readLogDistiller() {
    DOMConfigurator domConfigurator = new DOMConfigurator(properties);
    try {
      log("reading configuration " + conf);
      return domConfigurator.read(conf);
    } catch (IllegalArgumentException iae) {
      throw new BuildException("error in configuration: " + iae.getMessage(), iae);
    } catch (SAXException saxe) {
      throw new BuildException("parsing error in logdistiller XML configuration file " + conf, saxe);
    } catch (IOException ioe) {
      throw new BuildException("error while reading file " + conf, ioe);
    } catch (ParserConfigurationException pce) {
      throw new BuildException("error with XML parser configuration", pce);
    }
  }

  private FactoryMultiplexer initLogEventFactory(LogType.Description description) {
    // construct a log event multiplexer with each file added in the fileset
    FactoryMultiplexer multiplexer = new FactoryMultiplexer(description);
    try {
      int count = 0;
      Iterator iter = filesets.iterator();
      while (iter.hasNext()) {
        FileSet fs = (FileSet) iter.next();
        DirectoryScanner ds = fs.getDirectoryScanner(getProject());
        String[] srcFiles = ds.getIncludedFiles();
        int len = srcFiles.length;
        for (int i = 0; i < len; i++) {
          count += addFile(new File(ds.getBasedir(), srcFiles[i]), description, multiplexer);
        }
      }
      log("parsing " + count + " log files");
    } catch (IOException ioe) {
      throw new BuildException("error while opening log file", ioe);
    }
    return multiplexer;
  }

  private int addFile(File file, LogType.Description description, FactoryMultiplexer multiplexer) throws IOException {
    int count = 0;
    log("parsing " + file.getPath());
    if ("no".equals(compressed)) {
      multiplexer.addFactory(description.newFactory(new InputStreamReader(new FileInputStream(file)), file.toURL().toString()));
      count++;
    } else if ("gzip".equals(compressed)) {
      multiplexer.addFactory(description.newFactory(new InputStreamReader(new GZIPInputStream(new FileInputStream(file))),
          "gzip:" + file.toURL().toString()));
      count++;
    } else { // zip
      ZipFile zip = new ZipFile(file);
      Enumeration iter = zip.entries();
      while (iter.hasMoreElements()) {
        ZipEntry entry = (ZipEntry)iter.nextElement();
        if (! entry.isDirectory()) {
          log("   " + entry.getName());
          String logSource = "zip:" + file.toURL() + "!" + entry.getName();
          multiplexer.addFactory(description.newFactory(new InputStreamReader(zip.getInputStream(entry)), logSource));
          count++;
        }
      }
    }
    return count;
  }

  public static class Compressed extends EnumeratedAttribute {
      public String[] getValues() {
          return new String[] {"no", "gzip", "zip"};
      }
  }
}
