/*
 *  net/sf/logdistiller/gui/NewDialog.java
 *  Copyright (C) 2004-2007 Herv BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Frame;
import java.awt.LayoutManager;
import java.awt.event.ActionEvent;
import java.io.*;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.swing.*;

import com.jgoodies.forms.factories.ButtonBarFactory;
import com.jgoodies.forms.factories.DefaultComponentFactory;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;

import net.sf.logdistiller.LogType;
import net.sf.logdistiller.LogTypes;
import net.sf.logdistiller.util.FormatUtil;
import net.sf.logdistiller.util.PropertiesReplacer;

/**
 * Dialog to create a new rules configuration file: you select the log type and give some basic information
 * (an id, STMP server, url), and it generates a new rules configuration. Then you just need to refine
 * the rules.
 */
public class NewDialog {
  private File currentDir;
  public NewDialog(File directory) {
    currentDir = directory;
  }

  protected JPanel panel = new JPanel();
  private JTextField tfId = new JTextField("rules-id", 15);
  private JTextField tfDescription = new JTextField("rules description");
  private JComboBox cbLogtype;

  private JTextField tfSmtp = new JTextField("smtp.provider.com");
  private JTextField tfMailTo = new JTextField("admin@localhost");
  private JTextField tfHttp = new JTextField("http://mysite.com/logdistiller/logs");

  private JButton btSaveAs = new JButton("save as...");
  private JButton btCancel = new JButton("cancel");

  public static JComboBox createCbLogtype(boolean withUndefined) {
    List logtypes = LogTypes.getAllLogTypes();
    int len = logtypes.size();
    String[] logtypeIds = new String[len];
    for (int i = 0; i < len; i++) {
      logtypeIds[i] = ((LogType)logtypes.get(i)).getId();
    }
    JComboBox cb = new JComboBox(logtypeIds);
    if (withUndefined) {
      cb.insertItemAt("undefined", 0);
      cb.setSelectedIndex(0);
    }
    return cb;
  }

  private void initComponents() {
    cbLogtype = createCbLogtype(false);

    btSaveAs.addActionListener(SwingAdapter.getActionListener(this, "btSaveAsActionPerformed"));
    btCancel.addActionListener(SwingAdapter.getActionListener(this, "btCancelActionPerformed"));
  }

  private void addLabel(String label, String id) {
    panel.add(new JLabel(label), id);
  }

  private void addSeparator(String label, String id) {
    panel.add(DefaultComponentFactory.getInstance().createSeparator(label), id);
  }

  public LayoutManager createLayout(String layoutId) {
    return MainPanel.SIMPLE_LAYOUT_CONSTRAINTS_MANAGER.getLayout(layoutId);
  }

  public JComponent buildPanel() {
    initComponents();

    panel.setBorder(com.jgoodies.forms.factories.Borders.DIALOG_BORDER);
    panel.setLayout(createLayout("newDialog"));

    addSeparator("main parameters", "sepMain");
    addLabel("configuration id:", "lblId");
    panel.add(tfId, "tfId");
    addLabel("configuration description:", "lblDescription");
    panel.add(tfDescription, "tfDescription");
    addLabel("logtype:", "lblLogtype");
    panel.add(cbLogtype, "cbLogtype");

    addSeparator("advanced parameters", "sepOptions");
    addLabel("SMTP host:", "lblSmtp");
    panel.add(tfSmtp, "tfSmtp");
    addLabel("mail reports to:", "lblMailTo");
    panel.add(tfMailTo, "tfMailTo");
    addLabel("reports publication URL:", "lblHttp");
    panel.add(tfHttp, "tfHttp");

    Component bbar = ButtonBarFactory.buildRightAlignedBar(new JButton[] { btSaveAs, btCancel });
    panel.add(bbar, "bbar");

    return panel;
  }

  private JDialog createDialog(Component parent) {
    Frame frame = parent instanceof Frame ? (Frame) parent : (Frame)SwingUtilities.getAncestorOfClass(Frame.class, parent);
    JDialog dialog = new JDialog(frame, "new classification rules configuration", true);

    Container contentPane = dialog.getContentPane();
    contentPane.setLayout(new BorderLayout());
    contentPane.add(buildPanel(), BorderLayout.CENTER);

    dialog.pack();
    dialog.setLocationRelativeTo(parent);
    return dialog;
  }

  private JDialog dialog;
  public File showNewConfigurationDialog(Component parent) {
    dialog = createDialog(parent);
    dialog.show();
    dialog.dispose();
    dialog = null;
    return configurationFile;
  }

  private File configurationFile;
  public void btSaveAsActionPerformed(ActionEvent ae) {
    JFileChooser fc = new JFileChooser(currentDir);
    fc.setFileFilter(new XmlFileFilter());
    fc.setSelectedFile(new File(currentDir, tfId.getText() + ".xml"));
    int ret = fc.showSaveDialog(dialog);
    if (ret == JFileChooser.APPROVE_OPTION) {
      saveConfigurationFile(fc.getSelectedFile());
      configurationFile = fc.getSelectedFile();
      if (dialog != null) {
        dialog.setVisible(false);
      }
    }
  }

  /**
   * Saves a new rules configuration to a file.
   * @param fileTo the file to save to
   */
  private void saveConfigurationFile(File fileTo) {
    try {
      PrintWriter out = new PrintWriter(new OutputStreamWriter(new FileOutputStream(fileTo), "utf-8"));
      generateConfiguration(out);
      out.close();
    } catch (FileNotFoundException fnfe) {
      fnfe.printStackTrace();
    } catch (UnsupportedEncodingException uee) {
      uee.printStackTrace();
    }
  }

  /**
   * Generate a new configuration stream by filling <code>reference-rules.xml</code> with values
   * taken from the GUI.
   * @param out the stream to write to (in UTF-8 encoding)
   */
  private void generateConfiguration(PrintWriter out) {
    LogType logtype = LogTypes.getLogType((String) cbLogtype.getSelectedItem());
    LogType.Description description = logtype.newDescription(new HashMap());

    String logeventAttributes = FormatUtil.join(",", description.getAttributeNames());
    int dateAttributeIndex = description.getTimestampAttribute();

    try {
      String reference = IOUtils.toString(MainPanel.class.getResourceAsStream("reference-rules.xml"), "UTF-8");

      Map properties = new HashMap();
      properties.put("id", tfId.getText());
      properties.put("description", tfDescription.getText());
      properties.put("logtype", cbLogtype.getSelectedItem());
      properties.put("logtype.attributes", logeventAttributes);
      properties.put("mail.to", tfMailTo.getText());
      properties.put("mail.smtp.host", tfSmtp.getText());
      properties.put("logs.url", tfHttp.getText());
      properties.put("logtype.groups", description.getDefaultSpecificGroups());

      String skipCondition = // default matching condition for skip group, if no timestamp available
        "      <!-- this logtype does not have a timestamp: we can't really skip events with bad date -->\n" +
        "      <match attribute=\"logSource\" type=\"equals\"></match>\n" +
        "      <!-- if this logtype had a timestamp, the condition would look like the following: -->\n" +
        "      <!--match attribute=\"(timestamp attribute?)\" type=\"!contains\">${date}</match!-->"
      ;
      if (dateAttributeIndex >= 0) {
        skipCondition = "      <match attribute=\"" + description.getAttributeName(dateAttributeIndex) + "\" type=\"!contains\">${date}</match>";
      }
      properties.put("skip.condition", skipCondition);

      // let's compute the sampling.rule property for this LogType
      String[] samplingAttributes = StringUtils.split(description.getDefaultSamplingAttributes(), ",");
      String samplingRule = // default comment when no sampling attributes defined
        "    <!--plugin type=\"sampling\">\n" +
        "      <param name=\"attribute\">most discriminating attribute</param>\n" +
        "    </plugin-->";
      if (samplingAttributes.length > 0) {
        // ok, there are sampling attributes
        StringBuffer buff = new StringBuffer();

        if (samplingAttributes.length > 1) {
          // more than 1 sampling attribute => we add freq plugins
          for (int i = 1; i < samplingAttributes.length; i++) {
            buff.append(generatePluginEntry("freq", samplingAttributes, i));
          }
        }

        buff.append(generatePluginEntry("sampling", samplingAttributes, samplingAttributes.length));
        samplingRule = buff.toString();
      }
      properties.put("sampling.rule", samplingRule);

      PropertiesReplacer repl = new PropertiesReplacer(properties, "#(", ")");

      out.print(repl.replaceProperties(reference));
    } catch (IOException ioe) {
      ioe.printStackTrace();
    }
  }

  /**
   * Generate a plugin entry
   * @param plugin the plugin to use
   * @param attributes the attributes to configure
   * @param count the number of attributes to add
   * @return an XML fragment corresponding to the plugin entry
   */
  private static String generatePluginEntry(String plugin, String[] attributes, int count) {
    StringBuffer buff = new StringBuffer("\n    <plugin type=\"" + plugin + "\">\n");
    for (int i = 0; i < count; i++) {
      String n = (i == 0) ? "" : String.valueOf(i + 1);
      buff.append("      <param name=\"attribute" + n + "\">" + attributes[i] + "</param>\n");
    }
    buff.append("    </plugin>");
    return buff.toString();
  }

  public void btCancelActionPerformed(ActionEvent ae) {
    configurationFile = null;
    if (dialog != null) {
      dialog.setVisible(false);
    }
  }
}
