/*
 *  net/sf/logdistiller/logtypes/CSVLogEvent.java
 *  Copyright (C) 2004-2007 Herv� BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller.logtypes;

import java.io.*;
import java.text.ParseException;
import java.util.Arrays;
import java.util.Map;

import org.apache.commons.lang.StringUtils;

import net.sf.logdistiller.util.BufferingReader;
import net.sf.logdistiller.util.csv.CSVParse;
import net.sf.logdistiller.util.csv.CSVParser;

import net.sf.logdistiller.LogEvent;
import net.sf.logdistiller.LogType;

/**
 * A CSV (see <a href="http://tools.ietf.org/html/rfc4180">RFC-4180</a>) log parser, based on
 * <a href="http://ostermiller.org/utils/CSV.html">OstermillerUtils</a>. CSV log type supports following parameters:<ul>
 * <li><code>attributes</code>: specify the column names (and of course count), starting with <code>logSource</code></li>
 * <li><code>delimiter</code>: set the column delimiter (default value: <code>','</code>)</li>
 * <li><code>quote</code>: set the quoting character (default value: <code>'"'</code>)</li>
 * <li><code>commentStart</code>: set the list of characters a comment line may start with (default value: none)</li>
 * <!--li><code>escapes</code></li-->
 * </ul>
 * By default, the classification rules generated by the GUI for this type of logs won't be able to sort events, as the
 * log event structure is too generic.
 * @see CSVParser
 */
public class CSVLogEvent extends LogEvent {
  public final static String ID = "csv";
  public final static LogType LOGTYPE = new CSVLogType();

  public CSVLogEvent(Factory factory, String rawLog, String[] values) throws ParseException {
    super(factory, rawLog);
    int count = factory.getDescription().getAttributesCount();
    String[] attr = new String[count];
    Arrays.fill(attr, "");
    attr[0] = factory.getLogSource();
    System.arraycopy(values, 0, attr, 1, Math.min(values.length, count - 1));
    setAttributes(attr);
  }

  private static class CSVLogType extends LogType {
    public CSVLogType() {
      super(CSVLogEvent.ID);
    }

    public LogType.Description newDescription(Map params) {
      String attributes = (String)params.get("attributes");
      attributes = (attributes == null) ? "" : attributes;
      if (! attributes.startsWith("logSource,")) {
        attributes = "logSource," + attributes;
      }
      String[] attributeNames = StringUtils.split(attributes, ',');
      String attr = (String)params.get("delimiter");
      char delimiter = (attr == null) ? ',' : attr.charAt(0);
      attr = (String)params.get("quote");
      char quote = (attr == null) ? '"' : attr.charAt(0);
      String commentStart = (String)params.get("commentStart");
      commentStart = (commentStart == null) ? "" : commentStart;
      return new CSVLogEvent.Description(this, attributeNames, delimiter, quote, commentStart);
    }
  }

  private static class Description extends LogType.Description {
    private final char delimiter;
    private final char quote;
    private final String commentStart;
    public Description(LogType logtype, String[] attributeNames, char delimiter, char quote, String commentStart) {
      super(logtype, attributeNames);
      this.delimiter = delimiter;
      this.quote = quote;
      this.commentStart = commentStart;
    }

    public LogEvent.Factory newFactory(Reader reader, String logSource) throws IOException {
      return new Factory(this, reader, logSource);
    }

    public CSVParse newCSVParser(Reader reader) {
      CSVParser parser = new CSVParser(reader, delimiter);
      parser.changeQuote(quote);
      parser.setCommentStart(commentStart);
      return parser;
    }

    public int getTimestampAttribute() {
      return -1;
    }
  }

  private static class Factory extends LogEvent.Factory {
    private final BufferingReader buffering;
    private final CSVParse parser;

    public Factory(Description description, Reader reader, String logSource) throws FileNotFoundException {
      super(description, logSource);
      buffering = new BufferingReader(reader);
      this.parser = description.newCSVParser(buffering);
    }

    protected LogEvent readNextEvent() throws IOException, ParseException {
      int before = parser.getLastCharCount();
      String[] curLine = parser.getLine();
      if (curLine == null) {
        // EOF
        return null;
      }
      int after = parser.getLastCharCount();
      String rawLog = StringUtils.chomp(buffering.freeData(after - before));
      return new CSVLogEvent(this, rawLog, curLine);
    }
  }
}
