/*
 *  net/sf/logdistiller/logtypes/OracleAlertLogEvent.java
 *  Copyright (C) 2004-2007 Herv� BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller.logtypes;

import java.io.*;
import java.text.SimpleDateFormat;
import java.text.DateFormat;
import java.text.ParseException;
import java.util.Date;
import java.util.Locale;
import java.util.regex.Pattern;

import net.sf.logdistiller.LogEvent;
import net.sf.logdistiller.LogType;

/**
 * Oracle's <code>alert.log</code> file parser. A typical rule configuration with such log files is:
 * <pre>&lt;group id="ORA"&gt;
 *  &lt;description>ORA-* messages&lt;/description&gt;
 *  &lt;condition&gt;
 *    &lt;match attribute="message" type="contains"&gt;ORA-&lt;/match&gt;
 *  &lt;/condition&gt;
 *  &lt;plugin type="sampling"&gt;
 *    &lt;param name="attribute"&gt;message&lt;/param&gt;
 *    &lt;param name="regexp"&gt;(ORA-\d+)&lt;/param&gt;
 *  &lt;/plugin&gt;
 *&lt;/group&gt;</pre>
 * By default, the classification rules generated by the GUI for this type of logs: <ol>
 * <li>will contain a such "ORA" group for events with an ORA- id</li>
 * <li>but won't be able to sort other events, as the log event structure is too generic
 * (the message has no specific format).</li>
 * </ol>
 */
public class OracleAlertLogEvent extends LogEvent implements Comparable {
  public final static String ID = "oracle-alert";
  public final String timestamp;
  public final Date date;
  public final String message;

  public final static LogType LOGTYPE = new LogType.Basic(ID);
  private final static DateFormat DATE_FORMAT = new SimpleDateFormat("EEE MMM dd HH:mm:ss yyyy", Locale.US);
  private final static String[] ATTRIBUTE_NAMES = {
    "logSource",
    "timestamp", "timestamp.day", "timestamp.time",
    "message"
  };
  public final static LogType.Description DESCRIPTION = new Description((LogType.Basic)LOGTYPE, ATTRIBUTE_NAMES);

  public OracleAlertLogEvent(Factory factory, String firstLine, String message) throws ParseException {
    super(factory, firstLine + Factory.NEWLINE + message);
    timestamp = firstLine;
    date = DATE_FORMAT.parse(timestamp);
    this.message = message;
    setAttributes(new String[] {
        factory.getLogSource(),
        timestamp, timestamp.substring(0, 10) + timestamp.substring(19, 24), timestamp.substring(11, 19),
        message
    });
  }

  public int compareTo(Object o) {
    return date.compareTo(((OracleAlertLogEvent)o).date);
  }

  private static class Description extends LogType.Description {
    public Description(LogType.Basic logtype, String[] attributeNames) {
      super(logtype, attributeNames);
      logtype.setDescription(this);
    }

    public LogEvent.Factory newFactory(Reader reader, String logSource) throws IOException {
      return new Factory(this, reader, logSource);
    }

    public String getDefaultSpecificGroups() {
      return
        "  <group id=\"ORA\">\n" +
        "    <description>ORA-* messages</description>\n" +
        "    <condition>\n" +
        "      <match attribute=\"message\" type=\"contains\">ORA-</match>\n" +
        "    </condition>\n" +
        "    <report publisher=\"file\"/>\n" +
        "    <plugin type=\"sampling\">\n" +
        "      <param name=\"attribute\">message</param>\n" +
        "      <param name=\"regexp\">(ORA-\\d+)</param>\n" +
        "    </plugin>\n" +
        "  </group>";
    }
  }

  private static class Factory extends LogEvent.Factory {
    private final LineNumberReader reader;
    private String curLine;

    private final static Pattern DATE_PATTERN = Pattern.compile("\\w{3} \\w{3} \\d{2} \\d{2}:\\d{2}:\\d{2} \\d{4}");

    public Factory(Description description, Reader reader, String logSource) throws FileNotFoundException {
      super(description, logSource);
      this.reader = new LineNumberReader(reader);
    }

    /**
     * Detect the start of a new log event
     */
    protected boolean detectLogEventStart(String line) {
      return DATE_PATTERN.matcher(line).matches();
    }

    protected LogEvent readNextEvent() throws IOException, ParseException {
      if (curLine == null) {
        curLine = reader.readLine();
        if (curLine == null) {
          // EOF
          return null;
        }
        // beginning of log file does not start with a complete log event with timestamp
        while (! detectLogEventStart(curLine)) {
          curLine = reader.readLine();
          if (curLine == null) {
            // EOF
            return null;
          }
        }
      }
      String firstLine = curLine;
      StringBuffer buffer = new StringBuffer();
      while (((curLine = reader.readLine()) != null) && (! detectLogEventStart(curLine))) {
        if (buffer.length() > 0) {
          buffer.append(NEWLINE);
        }
        buffer.append(curLine);
      }
      return new OracleAlertLogEvent(this, firstLine, buffer.toString());
    }
  }
}
