/*
 *  net/sf/logdistiller/logtypes/SyslogLogEvent.java
 *  Copyright (C) 2004-2007 Herv BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller.logtypes;

import java.io.*;
import java.text.ParseException;

import net.sf.logdistiller.LogEvent;
import net.sf.logdistiller.LogType;
import net.sf.logdistiller.util.StringCutter;

/**
 * Log event for Unix syslog facility.
 * By default, the classification rules generated by the GUI for this type of logs will sort events
 * based on the following attributes: <code>host</code>, then <code>program</code>.
 */
public class SyslogLogEvent extends LogEvent {
  public final static String ID = "syslog";
  // see http://logreport.org/doc/gen/os/
  public final String timestamp;
  public final String host;
  public final String program;
  public final String pid;
  public final String message;

  public final static LogType LOGTYPE = new LogType.Basic(ID);
  private final static String[] ATTRIBUTE_NAMES = {
    "logSource",
    "timestamp",
    "host",
    "program",
    "pid",
    "message"
  };
  public final static LogType.Description DESCRIPTION = new Description((LogType.Basic)LOGTYPE, ATTRIBUTE_NAMES);

  public SyslogLogEvent(Factory factory, String rawLog) throws ParseException {
    super(factory, rawLog);
    timestamp = rawLog.substring(0, 15);
    StringCutter sc = new StringCutter(rawLog, 16);
    host = sc.parseTo(" ");
    String remaining = sc.getRemaining();
    if (remaining.indexOf(": ") < 0) {
      program = "";
      pid = "";
      message = remaining;
    } else {
      // <program>[<pid>]: <message>
      String programPid = sc.parseTo(": ");
      message = sc.getRemaining();

      int index = programPid.indexOf('[');
      if ((index >= 0) && programPid.endsWith("]")) {
        program = programPid.substring(0, index);
        pid = programPid.substring(index + 1, programPid.length() - 1);
      } else {
        program = programPid;
        pid = "";
      }
    }
    setAttributes(new String[] {
        factory.getLogSource(),
        timestamp,
        host,
        program,
        pid,
        message
    });
  }

  private static class Description extends LogType.Description {
    public Description(LogType.Basic logtype, String[] attributeNames) {
      super(logtype, attributeNames);
      logtype.setDescription(this);
    }

    public LogEvent.Factory newFactory(Reader reader, String logSource) throws IOException {
      return new Factory(this, reader, logSource);
    }

    public String getDefaultSamplingAttributes() {
      return "host,program";
    }
  }

  private static class Factory extends LogEvent.Factory {
    private final LineNumberReader reader;

    public Factory(Description description, Reader reader, String logSource) throws FileNotFoundException {
      super(description, logSource);
      this.reader = new LineNumberReader(reader);
    }

    protected LogEvent readNextEvent() throws IOException, ParseException {
      String curLine = reader.readLine();
      if (curLine == null) {
        // EOF
        return null;
      }
      try {
        return new SyslogLogEvent(this, curLine);
      } catch (RuntimeException re) {
        throw new RuntimeException("error while parsing line '" + curLine + "'", re);
      }
    }
  }
}
