/*
 *  net/sf/logdistiller/logtypes/WeblogicLogEvent.java
 *  Copyright (C) 2004-2007 Herv BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller.logtypes;

import java.io.*;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.Map;

import net.sf.logdistiller.LogEvent;
import net.sf.logdistiller.LogType;
import net.sf.logdistiller.util.FormatUtil;
import net.sf.logdistiller.util.StringCutter;

/**
 * Log event for <a href="http://edocs.bea.com/">BEA Weblogic server</a>'s logs.
 * By default, the date format is <code>MMM d, yyyy K:mm:ss a zz</code> with <code>Locale.US</code>, but
 * you can change these values with logtype parameters <code>date.format</code> and <code>date.locale</code>.
 * <br>Example:
 * <pre>&lt;logtype id="weblogic"&gt;
 *  &lt;param name="date.locale"&gt;fr_FR&lt;/param&gt;
 * &lt;/logtype&gt;</pre>
 * By default, the classification rules generated by the GUI for this type of logs will sort events
 * based on the following attributes: <code>severity</code>, then <code>subsystem</code> then <code>message_id</code>.
 */
public class WeblogicLogEvent extends LogEvent implements Comparable {
  public final static String ID = "weblogic";
  public final Date date;
  public final String timestamp;
  public final String severity;
  public final String subsystem;
  public final String machine;
  public final String server;
  public final String thread_id;
  public final String transaction_id;
  public final String user_id;
  public final String message_id;
  public final String message_text;
  public final String stacktrace;

  public final static LogType LOGTYPE = new WlLogType();
  private final static String LOGEVENT_START = "####<";
  private final static String[] ATTRIBUTE_NAMES = {
    "logSource",
    "timestamp",
    "severity",
    "subsystem",
    "machine",
    "server",
    "thread_id",
    "transaction_id",
    "user_id",
    "message_id",
    "message_text",
    "stacktrace"
  };

  public WeblogicLogEvent(Factory factory, String rawLog) throws ParseException {
    super(factory, rawLog);
    StringCutter cutter = new StringCutter(rawLog);
    cutter.parseTo(LOGEVENT_START);
    timestamp = cutter.parseTo("> <");
    date = factory.parseDate(timestamp);
    severity = cutter.parseTo("> <");
    subsystem = cutter.parseTo("> <");
    machine = cutter.parseTo("> <");
    server = cutter.parseTo("> <");
    thread_id = cutter.parseTo("> <");
    transaction_id = cutter.parseTo("> <");
    user_id = cutter.parseTo("> <");
    message_id = cutter.parseTo("> <");
    message_text = cutter.parseTo("> ");
    String remaining = cutter.getRemaining();
    stacktrace = ("".equals(remaining)) ? "" : remaining.substring(LogEvent.Factory.NEWLINE.length());
    setAttributes(new String[] {
        factory.getLogSource(),
        timestamp,
        severity,
        subsystem,
        machine,
        server,
        thread_id,
        transaction_id,
        user_id,
        message_id,
        message_text,
        stacktrace
    });
  }

  public int compareTo(Object o) {
    return date.compareTo(((WeblogicLogEvent)o).date);
  }

  /**
   * Weblogic log type supports following parameters:<ul>
   * <li><code>date.format</code></li>
   * <li><code>date.locale</code></li>
   * </ul>
   */
  private static class WlLogType extends LogType {
    private final static String DEFAULT_DATE_FORMAT = "MMM d, yyyy K:mm:ss a zz";

    public WlLogType() {
      super(WeblogicLogEvent.ID);
    }

    public LogType.Description newDescription(Map params) {
      String dateFormat = (String)params.get("date.format");
      dateFormat = (dateFormat == null) ? DEFAULT_DATE_FORMAT : dateFormat;
      String locale = (String)params.get("date.locale");
      Locale lcle = (locale == null) ? Locale.US : FormatUtil.parseLocale(locale);
      return new WeblogicLogEvent.Description(this, WeblogicLogEvent.ATTRIBUTE_NAMES, dateFormat, lcle);
    }
  }

  private static class Description extends LogType.Description {
    private final String format;
    private final Locale locale;
    private final DateFormat df;
    public Description(LogType logtype, String[] attributeNames, String format, Locale locale) {
      super(logtype, attributeNames);
      this.format = format;
      this.locale = locale;
      this.df = new SimpleDateFormat(format, locale);
    }

    public LogEvent.Factory newFactory(Reader reader, String logSource) throws IOException {
      return new Factory(this, reader, logSource);
    }

    public DateFormat getDateFormat() {
      return df;
    }

    public Locale getLocale() {
      return locale;
    }

    public String getFormat() {
      return format;
    }

    public String getDefaultSpecificGroups() {
      return
        "  <group id=\"warning\">\n" +
        "    <description>Warning events</description>\n" +
        "    <condition>\n" +
        "      <match attribute=\"severity\" type=\"equals\">Warning</match>\n" +
        "    </condition>\n" +
        "    <report publisher=\"file\"/>\n" +
        "    <plugin type=\"sampling\">\n" +
        "      <param name=\"attribute\">subsystem</param>\n" +
        "      <param name=\"attribute2\">message_id</param>\n" +
        "    </plugin>\n" +
        "  </group>\n" +
        "\n" +
        "  <group id=\"error\">\n" +
        "    <description>Error events</description>\n" +
        "    <condition>\n" +
        "      <match attribute=\"severity\" type=\"equals\">Error</match>\n" +
        "    </condition>\n" +
        "    <report publisher=\"file\"/>\n" +
        "    <plugin type=\"sampling\">\n" +
        "      <param name=\"attribute\">subsystem</param>\n" +
        "      <param name=\"attribute2\">message_id</param>\n" +
        "    </plugin>\n" +
        "  </group>";
    }

    public String getDefaultSamplingAttributes() {
      return "severity,subsystem,message_id";
    }
  }

  private static class Factory extends LogEvent.Factory {
    private final DateFormat df;
    private final LineNumberReader reader;
    private String curLine;

    public Factory(Description description, Reader reader, String logSource) throws FileNotFoundException {
      super(description, logSource);
      this.reader = new LineNumberReader(reader);
      this.df = description.getDateFormat();
    }

    protected boolean detectLogEventStart(String line) {
      return line.startsWith(LOGEVENT_START);
    }

    protected LogEvent readNextEvent() throws IOException, ParseException {
      if (curLine == null) {
        curLine = reader.readLine();
        if (curLine == null) {
          // EOF
          return null;
        }
      }
      if (! detectLogEventStart(curLine)) {
        if (curLine.length() > 15) {
          curLine = curLine.substring(0, 15) + "...";
        }
        String msg = " should start with '" + LOGEVENT_START + "' but found '" + curLine + "'";
        if (reader.getLineNumber() == 1) {
          throw new ParseException("bad log format: is it really a Weblogic log file? It" + msg, 0);
        }
        throw new ParseException("bad log format, line " + reader.getLineNumber() + msg, 0);
      }
      StringBuffer buffer = new StringBuffer(curLine);
      while (((curLine = reader.readLine()) != null) && (! detectLogEventStart(curLine))) {
        buffer.append(NEWLINE);
        buffer.append(curLine);
      }
      return new WeblogicLogEvent(this, buffer.toString());
    }

    public Date parseDate(String date) throws ParseException {
      try {
        return df.parse(date);
      } catch (ParseException pe) {
        Description desc = (Description)description;
        throw new ParseException(pe.getMessage() + ", expected format: " + desc.getFormat() + ", expected locale: " + desc.getLocale(), pe.getErrorOffset());
      }
    }
  }
}
