  /*
 *  net/sf/logdistiller/plugins/FreqPlugin.java
 *  Copyright (C) 2004-2007 Herv BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller.plugins;

import java.io.*;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import net.sf.logdistiller.LogDistillation;
import net.sf.logdistiller.LogDistiller;
import net.sf.logdistiller.LogEvent;
import net.sf.logdistiller.Plugin;
import net.sf.logdistiller.PluginConfigException;
import net.sf.logdistiller.ReportFormat;
import net.sf.logdistiller.util.FormatUtil;

/**
 * Counts the frequency of each attribute's value.
 * <p> Parameters:</p>
 * <ul>
 * <li><b>value</b>: definition of the value that will be taken into account
 * <ul>
 * <li><code>attribute</code> (mandatory): the attribute's name
 * <li><code>regexp</code>: use a regexp to extract value taken in account for the first attribute
 * <li><code>regexp.group</code> (default: -1): which group to extract? If negative, all groups will be concatenated
 * <li><code>attribute2</code> (default: none): second attribute to append
 * <li><code>attribute3</code> (default: none): third attribute to append
 * </ul></li>
 *
 * <li><b>sorting options</b><ul>
 * <li><code>sortItem</code> (default: <code>count</code>): sort on which item? <code>count</code>, <code>value</code>, <code>valueLength</code>
 * <li><code>reverseOrder</code> (default: <code>false</code>): by default, most frequent value are displayed first
 * </ul></li>
 *
 * <li><b>reporting</b><ul>
 * <li><code>valueDescriptions</code>: description to insert in the report instead of corresponding value (in
 * java properties format)
 * <li><code>maxGlobalReport</code> (default: 5): maximum number of values inserted in global report
 * <li><code>maxGroupReport</code> (default: 25): maximum number of values inserted in group report
 * </ul></li>
 *
 * </ul>
 *
 * @see SamplingPlugin
 */
public class FreqPlugin extends LogDistillation.Plugin {
  private static final long serialVersionUID = -5162563816094662936L;
  public final static String ID = "freq";
  public final static Plugin TYPE = new Type();
  private final static String[] SORT_ITEM_VALUES = {
      "count", "value", "valueLength"
  };
  private final static String[] SORT_ITEM_DESCRIPTIONS = {
      "count", "attribute's value", "attribute's value length"
  };
  private final static FreqOrder[] SORT_ITEM_COMPARATORS = {
      new CountOrder(), new ValueOrder(), new ValueLengthOrder()
  };

  private final Map freqs = new HashMap(); // key = attribute's value (String), value = Freq instance
  private SortedSet sorted; // sorted Freq
  private final String attribute;
  private final String attribute2;
  private final String attribute3;
  private int attributeIndex = -1;
  private int attributeIndex2 = -1;
  private int attributeIndex3 = -1;
  private final int sortItem; // index in SORT_ITEM_VALUES
  private final boolean reverseOrder;
  private final Properties valueDescriptions = new Properties();
  private final int maxGlobalReport;
  private final int maxGroupReport;
  private final String regexpString;
  private transient Pattern regexp;
  private final int regexpGroup;

  public FreqPlugin(LogDistiller.Plugin definition) {
  this(definition, true);
  }

  protected FreqPlugin(LogDistiller.Plugin definition, boolean checkSampling) {
    super(definition);
    attribute = definition.getParam("attribute");
    attribute2 = definition.getParam("attribute2", null);
    attribute3 = definition.getParam("attribute3", null);
    String attr = definition.getParam("sortItem");
    sortItem = (attr == null) ? 0 : Arrays.binarySearch(SORT_ITEM_VALUES, attr);
    attr = definition.getParam("reverseOrder", "false");
    reverseOrder = Boolean.valueOf(attr).booleanValue();
    attr = definition.getParam("valueDescriptions");
    if (attr != null) {
      try {
        valueDescriptions.load(new ByteArrayInputStream(attr.getBytes()));
      } catch (IOException ioe) {
        ioe.printStackTrace();
      }
    }
    maxGlobalReport = Integer.parseInt(definition.getParam("maxGlobalReport", "5"));
    maxGroupReport = Integer.parseInt(definition.getParam("maxGroupReport", "25"));
    regexpString = definition.getParam("regexp");
    updatePattern();
    regexpGroup = Integer.parseInt(definition.getParam("regexp.group", "-1"));

    if (checkSampling &&
        ((definition.getParam("sampling.maxCount") != null)
            || (definition.getParam("sampling.maxSize") != null)
            || (definition.getParam("sampling.filename") != null))) {
      definition.getGroup().getLogdistiller().addWarning("sampling features (sampling.* parameters) disabled from freq plugin: moved to sampling plugin");
    }
  }

  private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
    in.defaultReadObject();
    updatePattern();
  }

  private void updatePattern() {
    try {
      regexp = (regexpString == null) ? null : Pattern.compile(regexpString);
    } catch (PatternSyntaxException pse) {
      throw new PluginConfigException("regexp parameter contains bad pattern '" + regexpString + "': caused an exception "
                                      + pse.getMessage());
    }
  }

  public void begin(File destinationDirectory) throws FileNotFoundException {
  }

  private static int getAttributeIndex(LogEvent logEvent, String attribute) {
    if (attribute == null) {
      return -1;
    }
    int index = logEvent.getFactory().getDescription().getAttributeIndex(attribute);
    if (index < 0) {
      throw new PluginConfigException("attribute '" + attribute + "' unknown in logEvent, available values: "
                                      + FormatUtil.join(", ", logEvent.getFactory().getDescription().getAttributeNames()));
    }
    return index;
  }

  public void addLogEvent(LogEvent logEvent) throws IOException {
    addFreqLogEvent(logEvent);
  }

  protected Freq addFreqLogEvent(LogEvent logEvent) throws IOException {
    if (attributeIndex < 0) {
      attributeIndex = getAttributeIndex(logEvent, attribute);
      attributeIndex2 = getAttributeIndex(logEvent, attribute2);
      attributeIndex3 = getAttributeIndex(logEvent, attribute3);
    }
    String value = logEvent.getAttribute(attributeIndex);
    if (attributeIndex2 >= 0) {
      value += '-' + logEvent.getAttribute(attributeIndex2);
      if (attributeIndex3 >= 0) {
        value += '-' + logEvent.getAttribute(attributeIndex3);
      }
    }
    if (regexp != null) {
      Matcher matcher = regexp.matcher(value);
      if (matcher.find() && (regexpGroup <= matcher.groupCount())) {
        if (regexpGroup >= 0) {
          value = matcher.group(regexpGroup);
        } else if (matcher.groupCount() > 1) {
          StringBuffer buff = new StringBuffer(matcher.group(1));
          for (int i = 2; i <= matcher.groupCount(); i++) {
            buff.append('-').append(matcher.group(i));
          }
          value = buff.toString();
        } else if (matcher.groupCount() == 1) {
          value = matcher.group(1);
        } else {
          value = matcher.group(0);
        }
      } else {
        value = "";
      }
      if (value == null) {
        value = "";
      }
    }
    Freq freq = (Freq)freqs.get(value);
    long bytes = logEvent.getRawLog().length() + LogDistillation.LINE_SEPARATOR.length();
    if (freq == null) {
      value = value.intern();
      freq = new Freq(value, bytes);
      freqs.put(value, freq);
    } else {
      freq.count++;
      freq.bytes += bytes;
    }
    return freq;
  }

  public void end() throws IOException {
    FreqOrder comparator = SORT_ITEM_COMPARATORS[sortItem];
    comparator = reverseOrder ? new ReverseOrder(comparator) : comparator;
    sorted = new TreeSet(comparator);
    sorted.addAll(freqs.values());
  }

  public void appendGroupReport(ReportFormat.PluginReport report) {
    appendReport(report, maxGroupReport);
  }

  public void appendGlobalReport(ReportFormat.PluginReport report) {
    appendReport(report, maxGlobalReport);
  }

  SortedSet getSorted() {
    return sorted;
  }

  private String describeResult(int maxCount) {
    String count = ((maxCount < 0) || (maxCount >= sorted.size()))
        ? String.valueOf(sorted.size()) : (maxCount + "/" + sorted.size());
    String attrs = (attribute2 == null) ?
       "" : ("-'" + attribute2 + "'" + ((attribute3 == null) ? "" : ("-'" + attribute3 + "'")));
    return "values of " + (regexpString == null ? "" : "regexp on ") + "attribute '" + attribute + "'" + attrs + " sorted by" +
        (reverseOrder ? " reverse " : " ") + SORT_ITEM_DESCRIPTIONS[sortItem] + " (" + count + " different values)";
  }

  private void addItems(ReportFormat.PluginReport report, int maxCount) {
    Iterator iter = sorted.iterator();
    while (iter.hasNext() && ((maxCount < 0) || (maxCount-- > 0))) {
      Freq freq = (Freq)iter.next();
      report.addItem(freq.count, valueDescriptions.getProperty(freq.key, freq.key));
    }
  }

  private void appendReport(ReportFormat.PluginReport report, int maxCount) {
    report.beginPluginReport(this, describeResult(maxCount));
    addLinkIntoReport(report);
    addItems(report, maxCount);
    report.endPluginReport();
  }

  protected void addLinkIntoReport(ReportFormat.PluginReport report) {
  }

  static class Freq implements Serializable {
    private static final long serialVersionUID = 9195786567021952968L;
    public final String key;
    public int count = 1;
    public long bytes = 0;

    public Freq(String key, long bytes) {
      this.key = key;
      this.bytes = bytes;
    }
  }

  private abstract static class FreqOrder implements Comparator, Serializable {
  }

  private static class CountOrder extends FreqOrder {
    private static final long serialVersionUID = -933179294932120994L;

    public int compare(Object o1, Object o2) {
      Freq freq1 = (Freq)o1;
      Freq freq2 = (Freq)o2;
      int diff = freq2.count - freq1.count;
      return (diff == 0) ? freq1.key.compareTo(freq2.key) : diff;
    }
  }

  private static class ValueOrder extends FreqOrder {
    private static final long serialVersionUID = 1824757961838977443L;

    public int compare(Object o1, Object o2) {
      Freq freq1 = (Freq)o1;
      Freq freq2 = (Freq)o2;
      return freq1.key.compareTo(freq2.key);
    }
  }

  private static class ValueLengthOrder extends FreqOrder {
    private static final long serialVersionUID = 651766724239811116L;

    public int compare(Object o1, Object o2) {
      Freq freq1 = (Freq)o1;
      Freq freq2 = (Freq)o2;
      int diff = freq2.key.length() - freq1.key.length();
      return (diff == 0) ? freq1.key.compareTo(freq2.key) : diff;
    }
  }

  private static class ReverseOrder extends FreqOrder {
    private static final long serialVersionUID = 539304109704333616L;
    private final FreqOrder comparator;
    public ReverseOrder(FreqOrder comparator) {
      this.comparator = comparator;
    }

    public int compare(Object o1, Object o2) {
      return comparator.compare(o2, o1);
    }
  }

  private static class Type extends Plugin {
    public Type() {
      super(ID);
    }

    public LogDistillation.Plugin newInstance(LogDistiller.Plugin conf) {
      return new FreqPlugin(conf);
    }
  }
}
