/*
 *  net/sf/logdistiller/publishers/MailPublisher.java
 *  Copyright (C) 2004-2007 Herv BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller.publishers;

import java.io.*;
import java.util.Properties;
import javax.activation.DataHandler;
import javax.activation.FileDataSource;
import javax.mail.*;
import javax.mail.internet.*;

import net.sf.logdistiller.*;

/**
 * Send mail report of a LogDistillation. When the report is a group report,
 * a file containing logevents (if saved) is added as attachement. Global output
 * parameters are used as JavaMail session properties: the most important property
 * is <code>mail.smtp.host</code>, but see JavaMail documentation to find all
 * available properties.
 * <p> Parameters:<ul>
 * <li><code>to</code>, <code>cc</code> (at least one): recipients of the mail
 * <li><code>from</code> (default: <code>smtp.from</code> global output parameter, or <code>logdistiller@sourceforge.net</code>
 * <li><code>attachLogs</code> (default: <code>true</code>): add corresponding logevents as attachement
 * </ul>
 */
public class MailPublisher extends Publisher {
  public String getId() {
    return "mail";
  }

  public void publish(LogDistillation logdistillation, LogDistiller.Report report)
      throws IOException, PublishException {
    String content = logdistillation.getContent();
    String subject = "[logdistiller:" + logdistillation.getDefinition().getOutput().getId() + "]" + ((content == null) ? "" : (" " + content)) +
        " global report: " + logdistillation.getEventCount() + " logevents";
    ReportFormat format = ReportFormats.getReportFormat(report.getFormat());
    StringWriter sw = new StringWriter();
    format.report(logdistillation, sw);
    String body = sw.toString();
    try {
      sendMail(logdistillation, report, subject, body, format.getContentType(), null);
    } catch (MessagingException me) {
      throw new PublishException("error while sending global report by mail", me);
    }
  }

  public void publish(LogDistillation.Group group, LogDistiller.Report report) throws IOException, PublishException {
    LogDistillation logdistillation = group.getLogdistillation();
    String content = logdistillation.getContent();
    String subject = "[logdistiller:" + logdistillation.getDefinition().getOutput().getId() + "]" + ((content == null) ? "" : (" " + content)) +
        " report for group " + group.getDefinition().getId() + ": " + group.getEventCount() + " logevents";
    ReportFormat format = ReportFormats.getReportFormat(report.getFormat());
    StringWriter sw = new StringWriter();
    format.report(group, sw);
    String body = sw.toString();
    try {
      sendMail(logdistillation, report, subject, body, format.getContentType(), group.getLogFile());
    } catch (MessagingException me) {
      throw new PublishException("error while sending group report by mail", me);
    }
  }

  protected void sendMail(LogDistillation logdistillation, LogDistiller.Report report, String subject, String content,
                          String mimetype, File attach)
      throws MessagingException
  {
    LogDistiller ld = logdistillation.getDefinition();
    String to = report.getParam("to");
    String cc = report.getParam("cc");
    if ((to == null) && (cc == null)) {
      // nobody to send to: TODO add warning
      return;
    }
    Properties props = new Properties();
    props.putAll(ld.getOutput().getParams());
    Session session = Session.getDefaultInstance(props, null);
    MimeMessage msg = new MimeMessage(session);

    msg.setSubject(subject);
    Multipart mp = new MimeMultipart();
    BodyPart body = new MimeBodyPart();
    body.setContent(content, mimetype);
    mp.addBodyPart(body);
    msg.setContent(mp);
    if (to != null) {
      msg.setRecipients(Message.RecipientType.TO, InternetAddress.parse(to));
    }
    if (cc != null) {
      msg.setRecipients(Message.RecipientType.CC, InternetAddress.parse(cc));
    }
    String from = report.getParam("from", "logdistiller@sourceforge.net");
    msg.setFrom(new InternetAddress(from));

    String attachLogs = report.getParam("attachLogs", "true");
    if ((attach != null) && (attach.exists()) && ("true".equals(attachLogs))) {
      MimeBodyPart ap = new MimeBodyPart();
      FileDataSource fds = new FileDataSource(attach);
      ap.setDataHandler(new DataHandler(fds));
      ap.setFileName(attach.getName());
      mp.addBodyPart(ap);
    }

    sendMail(msg);
  }

  protected void sendMail(Message msg) throws MessagingException {
    Transport.send(msg);
  }
}
